#Import the basics
from pytriqs.archive import *
from pytriqs.gf.local import *
from pytriqs.operators.util.hamiltonians import *
from pytriqs.operators.util.U_matrix import *
import pytriqs.utility.mpi as mpi

#Import the DFT classes
from pytriqs.applications.dft.sumk_dft import *

#Import the Solver class
from pytriqs.applications.impurity_solvers.cthyb import *

filename = 'NiO'

# Now initialize the DFT class
SK = SumkDFT(hdf_file = filename+'.h5',use_dft_blocks = True)

# Setup the basic structure of the CTQMC Solver
beta = 40.0
n_orb = SK.corr_shells[0]['dim']
spin_names = ['up','down']
orb_names = [i for i in range(0,n_orb)]
gf_struct = SK.gf_struct_solver[0]

#Initialize the Solver class:
S = Solver(beta=beta, gf_struct=gf_struct)

# Setup the Hamiltonian
U = 4.0
J = 0.9
U4ind = U_matrix(l=2,U_int=U, J_hund=J, basis = 'cubic')
Umat_eg = eg_submatrix(U = U4ind)

H = h_int_slater(spin_names, orb_names, Umat_eg, off_diag = False)


# Parameters for the CTQMC Solver
p = {}
p["max_time"] = -1
p["random_name"] = ""
p["random_seed"] = 123 * mpi.rank + 567
p["length_cycle"] = 200
p["n_warmup_cycles"] = 10000
p["n_cycles"] = 100000

p["h_int"] = H

#This is for the tail:
p["fit_max_moment"] = 3
p["fit_min_n"] = 70
p["fit_max_n"] = 170
p["perform_tail_fit"] = True


# Now prepare the input for the Solver:
SK.put_Sigma(Sigma_imp = [S.Sigma_iw])
SK.calc_mu()
S.G_iw << SK.extract_G_loc()[0]
dm = S.G_iw.density()
# Set the Hartree Energy as initial guess for Sigma, using the density of Gloc
SK.calc_dc(dm, U_interact = U, J_hund = J, use_dc_formula = 0)
# now set Re Sigma to this value:
S.Sigma_iw <<= SK.dc_imp[0]['up'][0,0]


# We set up an archive for storage:
if mpi.is_master_node():
    ar = HDFArchive(filename + '.h5','a')
    if not 'DMFT_iterations' in ar: ar.create_group('DMFT_iterations')
    
# Now we do a loop:
n_it = 7

for it in range(n_it):
    
    mpi.report('======================\nDoing iteration: %s'%it)
    
    # Get G0
    S.G0_iw << inverse(S.Sigma_iw + inverse(S.G_iw))

    # Solve the impurity problem
    S.solve(**p)

    # save the output into the hdf5:
    if mpi.is_master_node():
         ar['DMFT_iterations']['Sigma_it'+str(it)] = S.Sigma_iw
         ar['DMFT_iterations']['G_it'+str(it)] = S.G_iw

         for name, sgf in S.Sigma_iw:
             mesh = np.array([pp for pp in sgf.mesh]).reshape(-1,1).imag
             re_data = sgf.data.real.reshape((sgf.data.shape[0],-1))
             im_data = sgf.data.imag.reshape((sgf.data.shape[0],-1))

             mesh_a_data = np.hstack((mesh,re_data,im_data))
             np.savetxt('Sigma_it' + str(it) + '_' + name + '.dat', mesh_a_data)


    
    # do some output
    dm = S.G_iw.density()
    densimp = S.G_iw.total_density()
    for sig,gf in S.G_iw:
        mpi.report("Orbital %s density: %.6f"%(sig,dm[sig][0,0].real))
    mpi.report('Total charge of Gimp : %.6f'%densimp)
 

    # Calculate double counting
    SK.calc_dc(dm, U_interact=U, J_hund=J, use_dc_formula = 0)

    # Get new Gloc, as above, but with symmetrisation
    SK.symm_deg_gf(gf_to_symm = S.Sigma_iw,orb=0)
    SK.put_Sigma(Sigma_imp = [S.Sigma_iw])
    SK.calc_mu()
    S.G_iw << SK.extract_G_loc()[0]

    # do some output:
    dm = S.G_iw.density()
    for sig,gf in S.G_iw:
        mpi.report("Orbital %s density: %.6f"%(sig,dm[sig][0,0].real))
    mpi.report('Total charge of Gloc : %.6f'%S.G_iw.total_density())

    if mpi.is_master_node(): 
        ar['DMFT_iterations']['iteration_count'] = it
        ar['DMFT_iterations']['chemical_potential'+str(it)] = SK.chemical_potential
        ar['DMFT_iterations']['dc_imp'+str(it)] = SK.dc_imp
        ar['DMFT_iterations']['densimp'+str(it)] = densimp
        ar['DMFT_iterations']['densloc'+str(it)] = S.G_iw.total_density()


if mpi.is_master_node():
    del ar
    
# Save to files
if mpi.is_master_node():
    
    for name, s in S.Sigma_iw:
        mesh = np.array([p for p in s.mesh]).reshape(-1,1).imag
        re_data = s.data.real.reshape((s.data.shape[0],-1))
        im_data = s.data.imag.reshape((s.data.shape[0],-1))

        mesh_a_data = np.hstack((mesh,re_data,im_data))
        np.savetxt('Sigma_DMFT_' + name + '.dat', mesh_a_data)


    for name, s in S.G_iw:
        mesh = np.array([p for p in s.mesh]).reshape(-1,1).imag
        re_data = s.data.real.reshape((s.data.shape[0],-1))
        im_data = s.data.imag.reshape((s.data.shape[0],-1))

        mesh_a_data = np.hstack((mesh,re_data,im_data))
        np.savetxt('G_DMFT_' + name + '.dat', mesh_a_data)

